const { 
  createInstance, 
  createModel, 
  supertest, 
  createClientDatabase, 
  createApplication 
} = require("./couchTestUtils")

describe("/models", () => {
  let request
  let server
  let app
  let instance

  beforeAll(async () => {
    ({ request, server } = await supertest())
    await createClientDatabase(request)
    app = await createApplication(request)
  });

  afterAll(async () => {
    server.close();
  })

  describe("create", () => {

    beforeEach(async () => {
      instance = await createInstance(request, app._id);
    });

    it("returns a success message when the model is successfully created", done => {
      request
        .post(`/api/${instance._id}/models`)
        .send({ 
          name: "TestModel",
          key: "name",
          schema: {
            name: { type: "string" }
          }
        })
        .set("Accept", "application/json")
        .expect('Content-Type', /json/)
        .expect(200)
        .end(async (err, res) => {
            expect(res.res.statusMessage).toEqual("Model TestModel created successfully.");            
            expect(res.body.name).toEqual("TestModel");            
            done();
        });
      })
    });

  describe("fetch", () => {
    let testModel

    beforeEach(async () => {
      instance = await createInstance(request, app._id)
      testModel = await createModel(request, instance._id, testModel)
    });

    it("returns all the models for that instance in the response body", done => {
      request
        .get(`/api/${instance._id}/models`)
        .set("Accept", "application/json")
        .expect('Content-Type', /json/)
        .expect(200)
        .end(async (_, res) => {
            const fetchedModel = res.body[0];
            expect(fetchedModel.name).toEqual(testModel.name);            
            expect(fetchedModel.type).toEqual("model");            
            done();
        });
      })
    });

  describe("destroy", () => {
    let testModel;

    beforeEach(async () => {
      instance = await createInstance(request, app._id)
      testModel = await createModel(request, instance._id, testModel)
    });

    it("returns a success response when a model is deleted.", done => {
      request
        .delete(`/api/${instance._id}/models/${testModel._id}/${testModel._rev}`)
        .set("Accept", "application/json")
        .expect('Content-Type', /json/)
        .expect(200)
        .end(async (_, res) => {
            expect(res.res.statusMessage).toEqual(`Model ${testModel._id} deleted.`);            
            done();
        });
      })
    });
});
